@@ -6,6 +6,9 @@
 import re
 import sys
 from BidirMap import BidirMap
+from logging import debug
+from S3 import S3
+from Utils import unicodise
 
 class S3Uri(object):
 	type = None
@@ -32,10 +35,13 @@ class S3Uri(object):
 	
 	def __str__(self):
 		return self.uri()
-	
+
+	def __unicode__(self):
+		return self.uri()
+
 	def public_url(self):
 		raise ValueError("This S3 URI does not have Anonymous URL representation")
-	
+
 class S3UriS3(S3Uri):
 	type = "s3"
 	_re = re.compile("^s3://([^/]+)/?(.*)", re.IGNORECASE)
@@ -45,7 +51,7 @@ class S3UriS3(S3Uri):
 			raise ValueError("%s: not a S3 URI" % string)
 		groups = match.groups()
 		self._bucket = groups[0]
-		self._object = groups[1]
+		self._object = unicodise(groups[1])
 
 	def bucket(self):
 		return self._bucket
@@ -63,12 +69,15 @@ class S3UriS3(S3Uri):
 		return "/".join(["s3:/", self._bucket, self._object])
 	
 	def public_url(self):
-		return "http://%s.s3.amazonaws.com/%s" % (self._bucket, self._object)
+		if S3.check_bucket_name_dns_conformity(self._bucket):
+			return "http://%s.s3.amazonaws.com/%s" % (self._bucket, self._object)
+		else:
+			return "http://s3.amazonaws.com/%s/%s" % (self._bucket, self._object)
 
 	@staticmethod
 	def compose_uri(bucket, object = ""):
 		return "s3://%s/%s" % (bucket, object)
-	
+
 class S3UriS3FS(S3Uri):
 	type = "s3fs"
 	_re = re.compile("^s3fs://([^/]*)/?(.*)", re.IGNORECASE)
@@ -78,7 +87,7 @@ class S3UriS3FS(S3Uri):
 			raise ValueError("%s: not a S3fs URI" % string)
 		groups = match.groups()
 		self._fsname = groups[0]
-		self._path = groups[1].split("/")
+		self._path = unicodise(groups[1]).split("/")
 
 	def fsname(self):
 		return self._fsname
@@ -97,7 +106,7 @@ class S3UriFile(S3Uri):
 		groups = match.groups()
 		if groups[0] not in (None, "file://"):
 			raise ValueError("%s: not a file:// URI" % string)
-		self._path = groups[1].split("/")
+		self._path = unicodise(groups[1]).split("/")
 
 	def path(self):
 		return "/".join(self._path)
